------------------------------------------------------------------------------
--                                                                          --
--                          AUNITSTUB COMPONENTS                            --
--                                                                          --
--      A U N I T S T U B . G E N E R A T O R . G A T H E R _ D A T A       --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2008-2011, AdaCore                     --
--                                                                          --
-- AUNITSTUB is  free  software;  you  can redistribute it and/or modify it --
-- under terms of the  GNU  General Public License as published by the Free --
-- Software  Foundation;  either  version  2, or (at your option) any later --
-- version.  AUNITSTUB  is  distributed in the hope that it will be useful, --
-- but  WITHOUT  ANY  WARRANTY;   without  even  the  implied  warranty  of --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details.  You should have received a copy of the --
-- GNU  General  Public License distributed with GNAT; see file COPYING. If --
-- not, write to the  Free  Software  Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02110-1301, USA.,                                      --
--                                                                          --
-- AUNITSTUB is maintained by AdaCore (http://www.adacore.com).             --
--                                                                          --
------------------------------------------------------------------------------

separate (AUnitStub.Stub_Generator)
procedure Gather_Data
  (The_Unit          :     Asis.Compilation_Unit;
   Data              : out Data_Holder;
   Apropriate_Source : out Boolean)
is

   Unit_SF_Name : constant String :=
     String'(Base_Name (To_String (Text_Name (The_Unit))));
   --  Stores the full name of the file containing the unit.

   Control : Traverse_Control := Continue;

   Dummy_State : No_State := Not_Used;

   --  Temporal elements used to maintain readability.
   Tmp_CU        : Asis.Compilation_Unit;
   Tmp_Element   : Asis.Element;
   Gen_Unit_Name : String_Access;

   Type_Counter : Positive := 1;

   --------------------------
   --  Inner Subprogramms  --
   --------------------------

   procedure First_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State);

   procedure Get_Records is new Traverse_Element
     (Pre_Operation     => First_Pre_Operation,
      Post_Operation    => No_Op,
      State_Information => No_State);
   --  Sets the vulue of Main_Type with the first tagged record element
   --  and checks that there's no more tagged records in the given unit.

   procedure Second_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State);

   procedure Get_Subprograms is new Traverse_Element
     (Pre_Operation     => Second_Pre_Operation,
      Post_Operation    => No_Op,
      State_Information => No_State);

   procedure Third_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State);

   procedure Get_Nested_Packages is new Traverse_Element
     (Pre_Operation     => Third_Pre_Operation,
      Post_Operation    => No_Op,
      State_Information => No_State);

   function Parent_Subtype_Unit_Original
     (Type_Decl  : Asis.Element;
      Is_Generic : Boolean) return Asis.Compilation_Unit;
   --  Equivalent to Enclosing_Compilation_Unit (Corresponding_Parent_Subtype)
   --  for most cases. In case of a generic tested package tries to treat
   --  parent declaration as if it was declared in a formal package.

   function Get_Nesting (Elem : Asis.Element) return String;
   --  Returns the full package prefix if the element.

--     function Is_Root_Nesting (Nesting : String; Encl_Unit : String)
--                               return Boolean;
--    --  Returns true for root level nesting (argument string does not contain
--     --  dots).

   function Parent_In_Same_Unit (Elem : Asis.Element) return Boolean;
   --  For a tagged record extention definition declared in a nested package
   --  checks if at least one of the corresponding parent types is declared
   --  in the root surrounding package.

   function Root_Level_Parent (Element : Asis.Element) return Asis.Element;
   --  Returns declaration of first predecessor of given tagged type that is
   --  located at the root package level.
   --  Returns Nil_Element if all predecessors are located in inner packages.

   -----------------
   -- Get_Nesting --
   -----------------

   function Get_Nesting (Elem : Asis.Element) return String is
      Res  : String_Access := new String'("");
      Buff : String_Access;

      Enclosing : Asis.Element;
   begin

      Enclosing := Enclosing_Element (Elem);

      loop

         exit when Is_Nil (Enclosing);

         if Res.all = "" then
            Free (Res);
            Res := new String'
              (To_String (Defining_Name_Image
               (First_Name (Enclosing))));
         else
            Buff :=
              new String'(To_String (Defining_Name_Image
                (First_Name (Enclosing))) &
                "." & Res.all);
            Free (Res);
            Res := new String'(Buff.all);
            Free (Buff);
         end if;

         Enclosing := Enclosing_Element (Enclosing);

      end loop;

      return Res.all;

   end Get_Nesting;

   ---------------------------
   --  First_Pre_Operation  --
   ---------------------------

   procedure First_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State)
   is
      pragma Unreferenced (State);

      Tmp_Element : Asis.Element;
      Cur_Element : Asis.Element := Element;

      Type_Data        : Base_Type_Info;
      Nested_Type_Data : Type_Info;

      Parent_Type_Check      : Asis.Element;
      Parent_Unit_Check      : Asis.Compilation_Unit;
      Parent_Unit_Check_Name : String_Access;

      Type_Doubling : Boolean := False;

      Not_Nested_Parent : Asis.Element;
--        Not_Nested_Parent_Info : Base_Type_Info;
      Index : Integer := 0;
   begin

      if Is_Private (Element) then
         Control := Abandon_Siblings;
         return;
      end if;

      if Elements.Element_Kind (Cur_Element) = A_Declaration then

         if
           Declaration_Kind (Cur_Element) = An_Ordinary_Type_Declaration or
           Declaration_Kind (Cur_Element) = A_Private_Type_Declaration   or
           Declaration_Kind (Cur_Element) = A_Private_Extension_Declaration
         then
            Tmp_Element := Type_Declaration_View (Cur_Element);

            case Definition_Kind (Tmp_Element) is
               when A_Tagged_Private_Type_Definition |
                    A_Private_Extension_Definition =>

                  Cur_Element := Corresponding_Type_Declaration (Cur_Element);
                  Tmp_Element := Type_Declaration_View (Cur_Element);

               when others =>
                  null;
            end case;

            case Type_Kind (Tmp_Element) is

               when A_Tagged_Record_Type_Definition |
                    A_Derived_Record_Extension_Definition =>

                  Type_Data.Main_Type_Elem := Cur_Element;

                  case Trait_Kind (Tmp_Element) is

                     when An_Abstract_Trait                 |
                          An_Abstract_Private_Trait         |
                          An_Abstract_Limited_Trait         |
                          An_Abstract_Limited_Private_Trait =>

                        Type_Data.Main_Type_Abstract := True;

                     when others =>
                        Type_Data.Main_Type_Abstract := False;
                        Data.Needs_Set_Up := True;

                  end case;

                  Type_Data.Main_Type_Text_Name := new
                    String'(To_String (Defining_Name_Image
                      (First_Name (Type_Data.Main_Type_Elem))));
                  Type_Data.Nesting := new String'(Get_Nesting (Cur_Element));
                  Type_Data.Parent_In_Same_Unit :=
                    Parent_In_Same_Unit (Cur_Element);
                  Type_Data.Has_Root_Nesting_Ancestor := False;

                  if
                    Type_Kind (Tmp_Element) =
                    A_Derived_Record_Extension_Definition
                  then
                     Parent_Type_Check :=
                       Parent_Type_Declaration (Cur_Element);

                     Type_Data.Argument_Father_Type_Name := new
                       String'(To_String (Defining_Name_Image
                         (First_Name (Parent_Type_Check))));

                     Type_Data.Argument_Father_Nesting := new
                       String'(Get_Nesting (Parent_Type_Check));

                     Parent_Unit_Check :=
                       Parent_Subtype_Unit_Original
                         (Cur_Element,
                          Data.Is_Generic);

                     Parent_Unit_Check_Name := new
                       String'(Base_Name (To_String
                         (Text_Name (Parent_Unit_Check))));

                     Type_Data.Argument_Father_Unit_Name := new
                       String'(To_String (Unit_Full_Name (Parent_Unit_Check)));

                     if
                       Type_Data.Argument_Father_Unit_Name.all /=
                         Type_Data.Argument_Father_Nesting.all
                     then
                        Not_Nested_Parent :=
                          Root_Level_Parent (Type_Data.Main_Type_Elem);
                        if Is_Nil (Not_Nested_Parent) then
                           Type_Data.Has_Root_Nesting_Ancestor := False;
                        else
                           Type_Data.Root_Nesting_Ancestor :=
                             new String'(To_String (Defining_Name_Image
                               (First_Name (Not_Nested_Parent))));
                           Type_Data.Has_Root_Nesting_Ancestor := True;
                        end if;
                     end if;

                     if Source_Present (Parent_Unit_Check_Name.all) then
                        Type_Data.Has_Argument_Father := True;
                     else
                        Type_Data.Has_Argument_Father := False;
                        Data.Needs_Fixtures := True;
                     end if;

                  else
                     Type_Data.Has_Argument_Father := False;
                     Data.Needs_Fixtures := True;
                  end if;

                  --  checking for doubled types.
                  --  should change to childreb abandoning for private parts
                  for I in Data.Type_Data_List.First_Index ..
                    Data.Type_Data_List.Last_Index
                  loop
                     if
                       Is_Equal
                         (Data.Type_Data_List.Element (I).Main_Type_Elem,
                          Cur_Element)
                     then
                        Type_Doubling := True;
                        exit;
                     end if;
                  end loop;

                  --  If the collected type is on the root nesting level
                  --  it should go to the root list, otherwise it should be
                  --  appended to the correspondig list of nested descendants.
                  if not Type_Doubling then

                     Type_Data.Type_Number := Type_Counter;
                     Type_Counter          := Type_Counter + 1;

                     if
                       Type_Data.Nesting.all = Data.Unit_Full_Name.all
                     then
                        Data.Type_Data_List.Append (Type_Data);
                     else
                        Not_Nested_Parent :=
                          Root_Level_Parent (Type_Data.Main_Type_Elem);
                        if Is_Nil (Not_Nested_Parent)
                          or else
                          Get_Nesting (Not_Nested_Parent) /=
                          Data.Unit_Full_Name.all
                        then
                           Nested_Type_Data := Type_Info (Type_Data);
                           Data.Nested_Type_Data_List.Append
                             (Nested_Type_Data);
                        else
                           Type_Data.Root_Nesting_Ancestor :=
                             new String'(To_String (Defining_Name_Image
                               (First_Name (Not_Nested_Parent))));
                           Type_Data.Has_Root_Nesting_Ancestor := True;

                           for I in Data.Type_Data_List.First_Index ..
                             Data.Type_Data_List.Last_Index
                           loop
                              if
                                Is_Equal
                                  (Data.Type_Data_List.Element (I).
                                     Main_Type_Elem,
                                   Not_Nested_Parent)
                              then
                                 Index := I;
                                 exit;
                              end if;
                           end loop;

                           --  A stub for now. To add the handling types
                           --  without root level parents.
                           if Index /= 0 then
                              Nested_Type_Data := Type_Info (Type_Data);
                              Type_Data := Data.Type_Data_List.Element (Index);

                              Type_Data.Nested_Descendants.Append
                                (Nested_Type_Data);
                              Data.Type_Data_List.Replace_Element
                                (Index, Type_Data);
                           end if;
                        end if;
                     end if;
                  end if;

                  Control := Abandon_Children;

               when others =>
                  null;
            end case;

         end if;

      end if;

   end First_Pre_Operation;

   -------------------------
   -- Parent_In_Same_Unit --
   -------------------------

   function Parent_In_Same_Unit (Elem : Asis.Element) return Boolean is
      Unit_Decl  : constant Asis.Declaration := Unit_Declaration (The_Unit);
      Parent     :          Asis.Element     := Elem;
      Has_Parent :          Boolean          := False;
   begin

      loop
         exit when Type_Kind (Type_Declaration_View (Parent)) /=
           A_Derived_Record_Extension_Definition;

         Parent := Parent_Type_Declaration (Parent);
         if Is_Equal (Enclosing_Element (Parent), Unit_Decl) then
            Has_Parent := True;
            exit;
         end if;
      end loop;

      return Has_Parent;

   end Parent_In_Same_Unit;

   ------------------------------------
   --  Parent_Subtype_Unit_Original  --
   ------------------------------------

   function Parent_Subtype_Unit_Original
     (Type_Decl  : Asis.Element;
      Is_Generic : Boolean) return Asis.Compilation_Unit
   is
      Dec_Elem : Asis.Element :=
        Parent_Type_Declaration (Type_Decl);

      Old_Unit : constant Asis.Compilation_Unit :=
        Enclosing_Compilation_Unit (Type_Decl);

      New_Unit : constant Asis.Compilation_Unit :=
        Enclosing_Compilation_Unit (Dec_Elem);
   begin

      if not Is_Equal (New_Unit, Old_Unit) then
         return New_Unit;
      end if;

      --  If the unit is the same, that means that parent subtype is declared
      --  in the formal package declaration. We need to get the declaration of
      --  the corresponding generic package.

      Dec_Elem := Enclosing_Element (Dec_Elem);

      if Is_Generic then
         return
           Enclosing_Compilation_Unit
             (Corresponding_Generic_Package (Dec_Elem));
      else
         return New_Unit;
      end if;

   exception

      when Asis.Exceptions.ASIS_Inappropriate_Element =>
         return Asis.Nil_Compilation_Unit;

   end Parent_Subtype_Unit_Original;

   -----------------------
   -- Root_Level_Parent --
   -----------------------

   function Root_Level_Parent (Element : Asis.Element) return Asis.Element is
      Elem : Asis.Element := Parent_Type_Declaration (Element);
   begin

      loop
         exit when Is_Nil (Elem);

         if
           Get_Nesting (Elem) =
           To_String (Unit_Full_Name (Enclosing_Compilation_Unit (Elem)))
         then
            return Elem;
         end if;

         Elem := Parent_Type_Declaration (Elem);
      end loop;

      return Asis.Nil_Element;
   end Root_Level_Parent;

   ----------------------------
   --  Second_Pre_Operation  --
   ----------------------------

   procedure Second_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State)
   is
      Subp       : Subp_Info;
      Owner_Def  : Asis.Element;
      Owner_Decl : Asis.Element;

      Type_Found : Boolean;

      Nesteds : Type_Info_List.List;
      Cur     : Type_Info_List.Cursor;
   begin
      pragma Unreferenced (State);

      if Is_Private (Element) then
         Control := Abandon_Siblings;
         return;
      end if;

      if Element_Kind (Element) = A_Declaration then

         case Declaration_Kind (Element) is

            when A_Protected_Type_Declaration   |
                 A_Single_Protected_Declaration =>
               Control := Abandon_Children;

            when A_Procedure_Declaration |
                 A_Function_Declaration =>

               Subp.Subp_Declaration := Element;
               Subp.Subp_Text_Name   :=
                 new String'(Get_Subp_Name (Element));
               Subp.Nesting := new String'(Get_Nesting (Element));

               case Trait_Kind (Element) is
                  when An_Abstract_Trait =>
                     Subp.Is_Abstract := True;
                  when others =>
                     Subp.Is_Abstract      := False;
                     Data.Needs_Assertions := True;
               end case;

               if Is_Dispatching_Operation (Element) then
                  Owner_Def  := Primitive_Owner (Element);
                  Owner_Decl := Enclosing_Element (Owner_Def);

                  case Definition_Kind (Owner_Def) is
                     when A_Private_Extension_Definition   |
                          A_Tagged_Private_Type_Definition =>
                        Owner_Decl :=
                          Corresponding_Type_Declaration (Owner_Decl);
                     when others =>
                        null;
                  end case;

                  Type_Found := False;
                  for
                    I in Data.Type_Data_List.First_Index ..
                         Data.Type_Data_List.Last_Index
                  loop

                     if
                       Is_Equal
                         (Data.Type_Data_List.Element (I).Main_Type_Elem,
                          Owner_Decl)
                     then
                        Subp.Corresp_Type :=
                          Data.Type_Data_List.Element (I).Type_Number;
                        Subp.Subp_Mangle_Name := new
                          String'(Mangle_Hash (Element,
                            Data.Type_Data_List.Element (I).Main_Type_Elem));
                        Type_Found := True;
                        exit;
                     else
                        Nesteds :=
                          Data.Type_Data_List.Element (I).Nested_Descendants;
                        Cur := Nesteds.First;
                        loop
                           exit when Cur = Type_Info_List.No_Element;

                           if
                             Is_Equal
                               (Type_Info_List.Element (Cur).Main_Type_Elem,
                                Owner_Decl)
                           then
                              Subp.Corresp_Type :=
                                Type_Info_List.Element (Cur).Type_Number;
                              Subp.Subp_Mangle_Name := new String'
                                (Mangle_Hash (Element,
                                 Type_Info_List.Element (Cur).Main_Type_Elem));
                              Type_Found := True;
                              exit;
                           end if;

                           Type_Info_List.Next (Cur);
                        end loop;
                     end if;

                  end loop;
               else
                  --  In simple case the type is always found, because in fact
                  --  we do not depend on it.
                  Type_Found            := True;
                  Subp.Corresp_Type     := 0;
                  Subp.Subp_Mangle_Name := new
                    String'(Mangle_Hash (Element, Asis.Nil_Element));
                  Data.Has_Simple_Case  := True;
                  Data.Needs_Fixtures   := True;
                  Data.Needs_Set_Up     := True;
                  Data.Needs_Assertions := True;
               end if;

               if not (Subp.Is_Abstract and not
                         Is_Dispatching_Operation (Element))
               then
                  if Type_Found then
                     Data.Subp_List.Append (Subp);
                  end if;
               end if;

               Control := Abandon_Children;

            when others =>
               null;

         end case;

      end if;

   end Second_Pre_Operation;

   -------------------------
   -- Third_Pre_Operation --
   -------------------------

   procedure Third_Pre_Operation
     (Element :        Asis.Element;
      Control : in out Traverse_Control;
      State   : in out No_State)
   is
      Package_Data : Package_Info;
   begin
      pragma Unreferenced (State);

      if Is_Private (Element) then
         Control := Abandon_Siblings;
         return;
      end if;

      if Element_Kind (Element) = A_Declaration then
         case Declaration_Kind (Element) is

            when A_Package_Declaration =>
               Package_Data.Name := new String'
                 (Get_Nesting (Element) & "." &
                  To_String (Defining_Name_Image (First_Name (Element))));
               Package_Data.Is_Generic := False;
               Package_Data.Data_Kind := Declaration_Data;
               Data.Package_Data_List.Append (Package_Data);

            when A_Generic_Package_Declaration =>
               Package_Data.Name := new String'
                 (Get_Nesting (Element) & "." &
                  To_String (Defining_Name_Image (First_Name (Element))));
               Package_Data.Is_Generic := True;
               Package_Data.Data_Kind := Declaration_Data;
               Data.Package_Data_List.Append (Package_Data);

            when A_Package_Instantiation =>
               Package_Data.Name := new String'
                 (Get_Nesting (Element) & "." &
                  To_String (Defining_Name_Image (First_Name (Element))));
               Package_Data.Data_Kind := Instantiation;
               Data.Package_Data_List.Append (Package_Data);
               Control := Abandon_Children;

            when others =>
               null;
         end case;

      end if;

   end Third_Pre_Operation;

begin

   case Declaration_Kind (Unit_Declaration (The_Unit)) is

      when A_Package_Declaration =>
         Data.Is_Generic := False;

      when A_Generic_Package_Declaration =>
         Data.Is_Generic := True;

      when A_Package_Instantiation =>
         if Get_Source_Status (Unit_SF_Name) = Waiting then
            Set_Source_Status (Unit_SF_Name, Pending);
            Apropriate_Source := False;
            return;
         end if;

         Tmp_Element := Normalize_Reference
           (Generic_Unit_Name (Unit_Declaration (The_Unit)));

         Tmp_CU := Enclosing_Compilation_Unit
           (Corresponding_Name_Declaration (Tmp_Element));

         Gen_Unit_Name :=
           new String'(Base_Name (To_String (Text_Name (Tmp_CU))));

         --  Checking if corresponding generic package is an argument package.
         if not Source_Present (Gen_Unit_Name.all) then
            Set_Source_Status (Unit_SF_Name, Bad_Inheritance);
            Apropriate_Source := False;
            Free (Gen_Unit_Name);
            return;
         end if;

         if Get_Source_Status (Gen_Unit_Name.all) /= Processed then
            Set_Source_Status (Unit_SF_Name, Bad_Inheritance);
            Apropriate_Source := False;
            Free (Gen_Unit_Name);
            return;
         end if;

         Data :=
           (Data_Kind      => Instantiation,
            Unit           => The_Unit,
            Unit_Full_Name =>
               new String'(To_String (Unit_Full_Name (The_Unit))),
            Unit_File_Name =>
               new String'(Base_Name (To_String (Text_Name (The_Unit)))),
            Gen_Unit => Tmp_CU,
            Gen_Unit_Full_Name  =>
               new String'(To_String (Unit_Full_Name (Tmp_CU))),
            Gen_Unit_File_Name  =>
               new String'(Gen_Unit_Name.all));

         Free (Gen_Unit_Name);
         Apropriate_Source := True;
         return;

      when others =>

         Report_Std
           ("aunitstub: "                                &
            Base_Name (To_String (Text_Name (The_Unit))) &
            " is an unsuported kind of unit.");

         Apropriate_Source := False;
         Set_Source_Status (Unit_SF_Name, Bad_Content);
         return;

   end case;

   Data.Unit           := The_Unit;
   Data.Unit_Full_Name := new String'(To_String (Unit_Full_Name (The_Unit)));
   Data.Unit_File_Name :=
     new String'(Base_Name (To_String (Text_Name (The_Unit))));

   Get_Nested_Packages (Unit_Declaration (The_Unit), Control, Dummy_State);

   Get_Records (Unit_Declaration (The_Unit), Control, Dummy_State);

   Get_Subprograms (Unit_Declaration (The_Unit), Control, Dummy_State);

   if Data.Type_Data_List.Is_Empty and Data.Subp_List.Is_Empty then
      Apropriate_Source := False;
      Set_Source_Status (Unit_SF_Name, Processed_In_Vain);
   else
      Apropriate_Source := True;
   end if;

end Gather_Data;
